=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2011-04-11 00:18:12 +0000
+++ bzrlib/branch.py	2011-04-12 00:44:43 +0000
@@ -1028,7 +1028,8 @@
             self.repository.fetch(source_repo, revision_id=revid)
         self.set_last_revision_info(revno, revid)
 
-    def import_last_revision_info_and_tags(self, source, revno, revid):
+    def import_last_revision_info_and_tags(self, source, revno, revid,
+                                           lossy=False):
         """Set the last revision info, importing from another repo if necessary.
 
         This is used by the bound branch code to upload a revision to
@@ -1038,6 +1039,10 @@
         :param source: Source branch to optionally fetch from
         :param revno: Revision number of the new tip
         :param revid: Revision id of the new tip
+        :param lossy: Whether to discard metadata that can not be
+            natively represented
+        :return: Tuple with the new revision number and revision id
+            (should only be different from the arguments when lossy=True)
         """
         if not self.repository.has_same_location(source.repository):
             try:
@@ -1049,6 +1054,7 @@
                 if_present_ids=tags_to_fetch).execute()
             self.repository.fetch(source.repository, fetch_spec=fetch_spec)
         self.set_last_revision_info(revno, revid)
+        return (revno, revid)
 
     def revision_id_to_revno(self, revision_id):
         """Given a revision id, return its revno"""

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2011-04-11 00:18:12 +0000
+++ bzrlib/commit.py	2011-04-12 00:44:43 +0000
@@ -454,8 +454,10 @@
             self._set_progress_stage("Uploading data to master branch")
             # 'commit' to the master first so a timeout here causes the
             # local branch to be out of date
-            self.master_branch.import_last_revision_info_and_tags(
-                self.branch, new_revno, self.rev_id)
+            (new_revno, self.rev_id) = self.master_branch.import_last_revision_info_and_tags(
+                self.branch, new_revno, self.rev_id, lossy=lossy)
+            if lossy:
+                self.branch.fetch(self.master_branch, self.rev_id)
 
         # and now do the commit locally.
         self.branch.set_last_revision_info(new_revno, self.rev_id)

=== modified file 'bzrlib/tests/per_branch/test_last_revision_info.py'
--- bzrlib/tests/per_branch/test_last_revision_info.py	2011-02-07 04:30:00 +0000
+++ bzrlib/tests/per_branch/test_last_revision_info.py	2011-04-12 00:44:43 +0000
@@ -62,10 +62,23 @@
         tree1.commit('1st post')
         revid = tree1.commit('2st post', allow_pointless=True)
         branch2 = self.make_branch('branch2')
-        branch2.import_last_revision_info_and_tags(tree1.branch, 2, revid)
+        self.assertEquals((2, revid),
+            branch2.import_last_revision_info_and_tags(tree1.branch, 2, revid))
         self.assertEqual((2, revid), branch2.last_revision_info())
         self.assertTrue(branch2.repository.has_revision(revid))
 
+    def test_import_lossy(self):
+        # importing with lossy=True works
+        tree1 = self.make_branch_and_tree('branch1')
+        tree1.commit('1st post')
+        revid = tree1.commit('2st post', allow_pointless=True)
+        branch2 = self.make_branch('branch2')
+        ret = branch2.import_last_revision_info_and_tags(tree1.branch, 2,
+            revid, lossy=True)
+        self.assertIsInstance(ret, tuple)
+        self.assertIsInstance(ret[0], int)
+        self.assertIsInstance(ret[1], str)
+
     def test_same_repo(self):
         # importing and setting last revision within the same repo
         tree = self.make_branch_and_tree('branch1')

